




from huggingface_hub import login

import pandas as pd
from sklearn.metrics import accuracy_score, precision_score, recall_score, f1_score
from datetime import datetime
import csv
from collections import Counter
import random
import numpy as np
import argparse
import re
import json

model_id = "unsloth_DeepSeek-R1-Distill-Qwen-32B-unsloth-bnb-4bit"

put hunging face token here
hf_token = ""
login(hf_token)



from unsloth import FastLanguageModel




dtype = None 
load_in_4bit = True
model, tokenizer = FastLanguageModel.from_pretrained(
    model_name = model_id,
    max_seq_length = 8196,
    dtype = dtype,
    load_in_4bit = load_in_4bit,
    token = hf_token, 
)



with open("test_claimdecomp_evidence_question_mapping_origin.json") as f:
  test_data = json.load(f)

def get_features_test(data):
    samples = []
    prompt = "Predict the verdict of a claim based on the provided question-answer pair evidence. Choose from the labels: 'True', 'False', 'Half True/False'. Disregard irrelevant question-answer pairs when assessing the claim. Justify your decision step by step using the provided evidence and select the appropriate label."
    
    system = """
    You are a fact-checker. You have done a fact-check to verify a Claim based on the provided question-answer pair evidence.

    Your task is to predict the verdict of a claim based on the provided question-answer pair evidence whether the Claim is one of the labels: 'True', 'False', 'Half True/False'. Do this by following:

    - Respond "True" only if the relevant evidence fully or almost fully supports and verifies the claim as correct.
    - Respond "False" if:
    - The relevant evidence contradicts or disproves the claim.
    - The claim is misleading based on the relevant evidence.
    - The evidence is too weak or insufficient to support the claim.
    - Respond "Half True/False" if the evidence is ambiguous, incomplete, or inconclusive, making it impossible to determine if the claim is fully true or false.

    Always adhere to the following rules:

    - Use information only from the recorded evidence: Avoid inserting information that is not implied by the evidence. You may use commonsense knowledge, though.
    - Avoid repeating yourself.
    """
    
    for fact in data:  # Giả sử bạn có một danh sách dữ liệu trong `data`
        claim = fact.get("claim", "")  # Lấy claim từ mỗi fact
        seen_docs = set()  # Tạo tập hợp để lưu các câu trả lời đã xuất hiện
        filtered_qa = []   # Danh sách mới chứa các Q/A hợp lệ
        for qa in fact.get("evidences", []):
            top_doc = qa['top_k_doc'][0] if qa['top_k_doc'] else ''
            
            if top_doc and top_doc in seen_docs:
                # print(top_doc)

                continue  # Bỏ qua Q và A nếu top_k_doc[0] đã xuất hiện
            
            seen_docs.add(top_doc)  # Đánh dấu tài liệu đã xuất hiện
            filtered_qa.append(qa)  # Thêm vào danh sách hợp lệ

        fact["input_str"] = (
            prompt + "\n\nClaim: " + claim + "\n\n" + "\n\n".join([
                f"Q{i+1}: {qa['questions']}\nA{i+1}: {qa['top_k_doc'][0] if qa['top_k_doc'] else 'No answer could be found.'}"
                for i, qa in enumerate(filtered_qa)
            ])
        )
        # print(fact["input_str"])
        
        sample = {}  # Khởi tạo sample nếu chưa có
        sample["system"] = system
        sample["prompt"] = fact["input_str"]
        sample["label"] = fact["label"]
        sample["claim"] = fact["claim"]
        samples.append(sample)
    
    return samples

test_features = get_features_test(test_data)
test_features = pd.DataFrame(test_features)
# test_features = test_features[35:36]

def generate_response(sample, model):

    inputs = tokenizer([sample['prompt']], return_tensors='pt').to("cuda")


    outputs = model.generate(
    input_ids=inputs.input_ids,
    attention_mask=inputs.attention_mask,
    max_new_tokens=3000,
    use_cache=False, temperature= 0.1
    )

    response = tokenizer.batch_decode(outputs)
    sample['response'] = response[0]
    if "Verdict:" in response[0]:
        response = response[0].split("Verdict:")[1]
    else:
        print("Không tìm thấy 'Verdict:' trong response.")
        print(response[0])
        response = ""  # Hoặc xử lý tùy ý

    if "Half True/False" in response:
        sample["prediction"] = "Conflicting"
    elif "False" in response:
        sample["prediction"] = "False"
    elif "True" in response:
        sample["prediction"] = "True"
    else:
        print("Error: ", response)
        sample["prediction"] = "Conflicting"
        # exit(0)
    return sample

def format_chat_template_test(example):
    
    example_json = [
        {"role": "system", "content": example["system"]},
         {"role": "user", "content": example["prompt"]}
                    ]

    
    example["prompt"] = tokenizer.apply_chat_template(example_json, tokenize=False, add_generation_prompt=True)
    return example

test_data = test_features.apply(format_chat_template_test, axis=1)

FastLanguageModel.for_inference(model)
from tqdm import tqdm

tqdm.pandas()  # Khởi tạo tqdm cho pandas

test_data = test_data.progress_apply(lambda sample: generate_response(sample, model), axis=1)


import os
from datetime import datetime
from pathlib import Path

model_id_clean = re.sub(r'[\\/#!?%:]', '_', model_id)  # Xử lý trước
saved_model_name = "model"

# Lấy thời gian hiện tại và định dạng thành chuỗi
current_time = datetime.now().strftime("%Y-%m-%d_%H-%M-%S")



log_data = test_data

# Chuyển thành DataFrame
df = pd.DataFrame(test_data)

# Lưu DataFrame vào CSV
output_file = f"log_{model_id_clean}.csv"
df.to_csv(output_file, index=False)

print(f"log saved to {output_file}")


actual = test_data['label']
prediction = test_data['prediction']

from sklearn.metrics import confusion_matrix
import matplotlib.pyplot as plt
import seaborn as sns



def run_metrics( actual, prediction):
    """Run the metrics based on the loaded config."""
    print("Running metrics with the following settings:")

    # Calculate metrics
    accuracy = accuracy_score(actual, prediction)
    m_f1 = f1_score(actual, prediction, average='macro')  # Use 'macro' for multi-class
    w_f1 = f1_score(actual, prediction, average='weighted') 
    metrics = {
        "W-F1 Score": w_f1,
        "M-F1 Score": m_f1,
        "Accuracy": accuracy
    }
    cm = confusion_matrix(actual, prediction)

    labels = ["Conflicting", "False", "True"]

    # Plot confusion matrix
    plt.figure(figsize=(6,5))
    sns.heatmap(cm, annot=True, fmt='d', cmap="Blues", xticklabels=labels, yticklabels=labels)
    plt.xlabel("Predicted Label")
    plt.ylabel("True Label")
    plt.title("Confusion Matrix")
    plt.savefig(f"result_metrics_confusion_matrix.png")
    plt.show()

    # Check each metric and print if it's enabled in config
    result_data = {}
    print(metrics)
    # Check each metric and store if it's enabled in config
  
    result_data['Weighted F1 Score'] = metrics['W-F1 Score']

    result_data['Macro F1 Score'] = metrics['M-F1 Score']

    result_data['Accuracy'] = metrics['Accuracy']

    output_file = f"result_metrics_temp_score.csv"
    with open(output_file, 'w', newline='', encoding='utf-8') as file:
        writer = csv.DictWriter(file, fieldnames=result_data.keys())
        writer.writeheader()  # Write the header
        writer.writerow(result_data)  # Write the metrics

    print(f"Metrics saved to {output_file}")

run_metrics(actual, prediction)
